import { createSerialOperation } from '@deep-foundation/deeplinks/imports/gql/index.js';
import { getAllDeviceInfo } from './get-all-device-info.js';
/**
 * Saves device information by modifying the value of the device link.
 * Any fields not included in the `info` parameter will not be modified.
 * If the {@link SaveDeviceInfoParam.info} parameter is undefined, the function will call {@link getAllDeviceInfo} to get the device information.
 *
 * @example
 * #### Save all device information
```ts

import { saveDeviceInfo, getAllDeviceInfo } from '@deep-foundation/capacitor-device';

// Implicitly (if you do not pass info to saveDeviceInfo - it will save all information by default)
await saveDeviceInfo({deep, deviceLinkId});

// Explicitly
const allDeviceInfo = await getAllDeviceInfo();
await saveDeviceInfo({deep, deviceLinkId, info: allDeviceInfo});
```
 * #### Save specific device information
```ts
import { saveDeviceInfo } from '@deep-foundation/capacitor-device';

const deviceBatteryInfo = await Device.getBatteryInfo();
await saveDeviceInfo({deep, deviceLinkId, info: deviceBatteryInfo});

const {value: languageCode} = await Device.getLanguageCode();
await saveDeviceInfo({deep, deviceLinkId, info: {languageCode}});

const {value: languageTag} = await Device.getLanguageTag();
await saveDeviceInfo({deep, deviceLinkId, info: {languageTag}});
```
#### Save information manually
```ts
const myDeviceData = {
  "identifier": "d787dc6b-041f-4a61-8078-d5573c2e7a4e",
  "model": "X11",
  "platform": "web",
  "isVirtual": false,
  "osVersion": "x86.64",
  "isCharging": true,
  "languageTag": "en",
  "batteryLevel": 1,
  "languageCode": "en",
  "manufacturer": "Google Inc.",
  "webViewVersion": "113.0.0.0",
  "operatingSystem": "unknown"
};

await saveDeviceInfo({deep, deviceLinkId, info: myDeviceData});
```
 *
 */
export async function saveDeviceInfo(param) {
    const { deep, info: data } = param;
    const deviceLink = await getDeviceLink();
    const value = await getValue({
        deviceLink,
        data,
    });
    const serialOperations = await getSerialOperations({
        deviceLink,
        value,
    });
    await deep.serial({
        operations: serialOperations,
    });
    async function getDeviceLink() {
        let deviceLink;
        if ('deviceLinkId' in param) {
            if (!param.deviceLinkId) {
                throw new Error(`deviceLinkId is undefined`);
            }
            const { data } = await deep.select({
                id: param.deviceLinkId,
            });
            deviceLink = data[0];
        }
        else if ('deviceLink' in param) {
            if (!param.deviceLink) {
                throw new Error(`deviceLink is undefined`);
            }
            deviceLink = param.deviceLink;
        }
        else {
            throw new Error(`Either deviceLink or deviceLinkId must be passed`);
        }
        return deviceLink;
    }
    async function getValueInsertSerialOperation({ deviceLink, value, }) {
        return createSerialOperation({
            table: 'objects',
            type: 'insert',
            objects: {
                link_id: deviceLink.id,
                value: value,
            },
        });
    }
    async function getValueUpdateSerialOperation({ deviceLink, value, }) {
        return createSerialOperation({
            table: 'objects',
            type: 'update',
            exp: {
                link_id: deviceLink.id,
            },
            value: {
                value: value,
            },
        });
    }
    async function getValue({ deviceLink, data, }) {
        return {
            ...(deviceLink.value?.value ?? {}),
            ...(data ?? (await getAllDeviceInfo())),
        };
    }
    async function getSerialOperations({ deviceLink, value, }) {
        let serialOperations = [];
        if (!deviceLink.value) {
            serialOperations.push(await getValueUpdateSerialOperation({
                deviceLink,
                value,
            }));
        }
        else {
            serialOperations.push(await getValueInsertSerialOperation({
                deviceLink,
                value,
            }));
        }
        return serialOperations;
    }
}
